#!/bin/sh

ACTION=${1}
shift 1

get_image() {
  IMAGE_NAME=$(uci get seafile.@seafile[0].image_name 2>/dev/null)
}

do_install_detail() {
  local config=$(uci get seafile.@seafile[0].config_path 2>/dev/null)
  local port=$(uci get seafile.@seafile[0].port 2>/dev/null)
  local IMAGE_NAME=$(uci get seafile.@seafile[0].image_name 2>/dev/null)
  local username=$(uci get seafile.@seafile[0].username 2>/dev/null)
  local password=$(uci get seafile.@seafile[0].password 2>/dev/null)

  GEN_PASS=$(< /dev/urandom tr -dc A-Za-z0-9 2>/dev/null | head -c14; echo)
  GEN_PASS2=$(< /dev/urandom tr -dc A-Za-z0-9 2>/dev/null | head -c14; echo)

  LAN_IP=$(uci get network.lan.ipaddr)
  LAN_IP="${LAN_IP%/*}"

  [ -z "$config" ] && { echo "config path is empty!"; exit 1; }
  [ -z "$port" ] && port=8160
  [ -z "$IMAGE_NAME" ] && IMAGE_NAME="seafileltd/seafile-mc:latest"

  rm -R /opt/docker2/compose/seafile-app
  mkdir /opt/docker2/compose/seafile-app

  create_docker_compose_file
  docker_compose_action
  configure_shortcut_menu
}

create_docker_compose_file() {
  touch /opt/docker2/compose/seafile-app/docker-compose.yml
  cat > /opt/docker2/compose/seafile-app/docker-compose.yml <<EOF
version: '2.0'
services:
  db:
    image: mariadb:10.5
    container_name: seafile-mysql
    environment:
      - MYSQL_ROOT_PASSWORD=${GEN_PASS}  # Requested, set the root's password of MySQL service.
      - MYSQL_LOG_CONSOLE=true
    volumes:
      - ./seafile-mysql/db:/var/lib/mysql  # Requested, specifies the path to MySQL data persistent store.
    networks:
      - seafile-net

  memcached:
    image: memcached:1.6
    container_name: seafile-memcached
    entrypoint: memcached -m 256
    networks:
      - seafile-net

  seafile:
    image: seafileltd/seafile-mc:latest
    container_name: seafile
    ports:
      - "${port}:80"
#     - "4643:443"  # If https is enabled, cancel the comment.
    volumes:
      - ./seafile-data:/shared   # Requested, specifies the path to Seafile data persistent store.
    environment:
      - DB_HOST=db
      - DB_ROOT_PASSWD=${GEN_PASS}  # Requested, the value shuold be root's password of MySQL service.
      - TIME_ZONE=Etc/UTC  # Optional, default is UTC. Should be uncomment and set to your local time zone.
      - SEAFILE_ADMIN_EMAIL=me@example.com # Specifies Seafile admin user, default is 'me@example.com'.
      - SEAFILE_ADMIN_PASSWORD=asecret     # Specifies Seafile admin password, default is 'asecret'.
      - SEAFILE_SERVER_LETSENCRYPT=false   # Whether to use https or not.
#     - SEAFILE_SERVER_HOSTNAME=docs.seafile.com # Specifies your host name if https is enabled.
    depends_on:
      - db
      - memcached
    networks:
      - seafile-net

networks:
  seafile-net:
    labels:
      plugsy.name: "Seafile"
      plugsy.category: "Home"
      plugsy.icon: "@styled-icons/icomoon/Seafile"
      plugsy.link: "http://${LAN_IP}:${port}"
EOF
}

docker_compose_action() {
  docker-compose -f /opt/docker2/compose/seafile-app/docker-compose.yml up -d
}

configure_shortcut_menu() {
  uci add shortcutmenu lists
  uci set shortcutmenu.@lists[-1].webname="$IMAGE_NAME"
  uci set shortcutmenu.@lists[-1].weburl="$LAN_IP:$port"
  uci set shortcutmenu.@lists[-1].webpath="/"
  uci commit shortcutmenu
}

usage() {
  echo "usage: $0 sub-command"
  echo "where sub-command is one of:"
  echo "      install                Install seafile"
  echo "      upgrade                Upgrade seafile"
  echo "      rm/start/stop/restart  Remove/Start/Stop/Restart seafile"
  echo "      status                 Seafile status"
  echo "      port                   Seafile port"
}

case ${ACTION} in
  "install"|"upgrade")
    get_image
    do_install_detail
  ;;
  "rm")
    get_image
    CONTAINER_IDS=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.ID}}')
    echo "Stopping and removing containers..."
    for ID in $CONTAINER_IDS; do
      docker stop $ID
      docker rm $ID
    done
    docker rmi -f $IMAGE_NAME
  ;;
  "start"|"stop"|"restart")
    get_image
    CONTAINER_IDS=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.ID}}')
    for ID in $CONTAINER_IDS; do
      docker ${ACTION} ${ID}
    done
  ;;
  "status")
    get_image
    CONTAINER_NAMES=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.Names}}')
    docker ps --all -f "name=${CONTAINER_NAMES}" --format '{{.Status}}'
  ;;
  "port")
    get_image
    CONTAINER_NAMES=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.Names}}')
    docker ps --all -f "name=${CONTAINER_NAMES}" --format '{{.Ports}}' | grep -om1 '0.0.0.0:[0-9]*' | sed 's/0.0.0.0://'
  ;;
  *)
    usage
    exit 1
  ;;
esac
